@@ -245,22 +245,56 @@ def train_val_split(params, data, df, labels=None):
     split_index = params["id_field"]
     # Split data randomly between train and validation
     data["id_all"] = df[split_index].drop_duplicates().sort_values()
+    today = datetime.utcnow()
 
     if params["val_split_timeseries"]:
         logger.info("Time based separation of training and validation")
-        val_split_date = datetime.today() - timedelta(days=params["val_days"])
 
-        logger.info("Separating ids")
-        data["id_train"] = list(
-            df[split_index][
-                pd.to_datetime(
-                    df[params["train_cutoff_field"]],
-                    utc=True,
-                    infer_datetime_format=True,
-                )
-                < pd.to_datetime(val_split_date, utc=True, infer_datetime_format=True)
-            ].unique()
+        if "validation_test_split" in params and params["validation_test_split"]:
+            logger.info("Separating train-val-validation_test ids")
+
+            date_time_diff = today - timedelta(days=params["initial_days"])
+
+            train_split = date_time_diff + timedelta(days=params["train_days"])
+            val_split = pd.to_datetime(train_split) + timedelta(days=params["val_days"])
+
+            train_split_date = pd.to_datetime(
+                train_split, utc=True, infer_datetime_format=True
+            )
+
+            val_split_date = pd.to_datetime(
+                val_split, utc=True, infer_datetime_format=True
+            )
+
+            ind_test = (
+                df[["shipment_id", "upload_date", "service_completion_estimated_date"]]
+                .sort_values("upload_date")
+                .groupby("shipment_id")
+                .last()["service_completion_estimated_date"]
+                .between(train_split_date, val_split_date)
+            )
+
+            data["id_val"] = ind_test[ind_test].index
+
+        else:
+            train_split = datetime.today() - timedelta(
+                days=params["val_days"]
+            )  # 150 days
+
+            train_split_date = pd.to_datetime(
+                train_split, utc=True, infer_datetime_format=True
+            )
+
+        ind_train = (
+            df[["shipment_id", "upload_date", "service_completion_estimated_date"]]
+            .sort_values("upload_date")
+            .groupby("shipment_id")
+            .last()["service_completion_estimated_date"]
+            < train_split_date
         )
+
+        data["id_train"] = ind_train[ind_train].index
+
     else:
         logger.info("Random separation of training and validation")
         logger.info("Separating ids")
@@ -271,10 +305,19 @@ def train_val_split(params, data, df, labels=None):
             )
             .values
         )
-    data["id_val"] = list(set(data["id_all"]) - set(data["id_train"]))
+
+    if "validation_test_split" in params and params["validation_test_split"]:
+
+        data["id_test"] = list(
+            set(data["id_all"]) - set(data["id_train"]) - set(data["id_val"])
+        )
+        segments = ["train", "val", "test"]
+    else:
+        data["id_val"] = list(set(data["id_all"]) - set(data["id_train"]))
+        segments = ["train", "val"]
 
     # Loop over train and validation segments
-    for segment in ["train", "val"]:
+    for segment in segments:
         logger.info(f"Extracting {segment} data")
         # Select the index of a selected shipments of a segment
         idx = df[split_index].isin(data[f"id_{segment}"])
@@ -408,7 +451,11 @@ def data_etl(params, df, labels=None, **kwargs):
             ft_encoding=params["ft_encoding"],
         )
 
-        for segment in ["train", "val"]:
+        if "validation_test_split" in params and params["validation_test_split"]:
+            segments = ["train", "val", "test"]
+        else:
+            segments = ["train", "val"]
+        for segment in segments:
             # Do label gathering first since you need it for LDA
             data[f"label_{segment}"] = (
                 data[f"id_{segment}"]
