@@ -271,6 +271,7 @@ def model_evaluation(params, data):
     if params["prediction_type"] == "classification":
         results = calculate_mean_average_precision(params, data)
         results.update(kappa_analysis(params, data))
+        results.update(calculate_lift_curve(params, data))
 
     elif params["prediction_type"] == "regression":
         results = calculate_mean_absolute_error(params, data)
@@ -287,11 +288,24 @@ def ml_performance(params, model, data):
     """Model-agnostic performance evaluation."""
     # Train mode
     if params["train_mode"]:
+        # Compute prediction on training data
+        df_pred = ml_predict(params, model, data["df_train"])
+        data["pred_train"] = pd.DataFrame(df_pred, columns=params["data_output_fields"])
+        data["pred_train"].index = data["id_train"][params["id_field"]]
+
         # Compute prediction on validation data
         df_pred = ml_predict(params, model, data["df_val"])
         data["pred_val"] = pd.DataFrame(df_pred, columns=params["data_output_fields"])
         data["pred_val"].index = data["id_val"][params["id_field"]]
 
+        # Compute prediction on test data if exists
+        if "validation_test_split" in params and params["validation_test_split"]:
+            df_pred = ml_predict(params, model, data["df_test"])
+            data["pred_test"] = pd.DataFrame(
+                df_pred, columns=params["data_output_fields"]
+            )
+            data["pred_test"].index = data["id_test"][params["id_field"]]
+
         results = model_evaluation(params, data)
 
         if params["plot"]:
@@ -548,3 +562,50 @@ def performance_lift(params, data, model):
     compute_performance_table(params, data["check_total"], mode="total")
 
     return
+
+
+def calculate_single_lift_curve(y, y_pred, step=0.01):
+    """Calculate single Lift Curve."""
+    if len(y.shape) > 1:
+        return [calculate_single_lift_curve(y[c], y_pred[c], step) for c in y.columns]
+
+    # Define an auxiliary dataframe to plot the curve
+    aux_lift = pd.DataFrame()
+    # Create a real and predicted column for our new DataFrame and assign values
+    aux_lift["real"] = y
+    aux_lift["predicted"] = y_pred
+    # Order the values for the predicted probability column:
+    aux_lift.sort_values("predicted", ascending=False, inplace=True)
+
+    # Create the values that will go into the X axis of our plot
+    x_val = np.arange(step, 1 + step, step)
+    # Calculate the ratio of ones in our data
+    ratio_ones = aux_lift["real"].sum() / len(aux_lift)
+    # Create an empty vector with the values that will go on the Y axis our our plot
+    y_v = []
+
+    # Calculate for each x value its correspondent y value
+    for x in x_val:
+        num_data = int(
+            np.ceil(x * len(aux_lift))
+        )  # The ceil function returns the closest integer bigger than our number
+        data_here = aux_lift.iloc[:num_data, :]  # ie. np.ceil(1.4) = 2
+        ratio_ones_here = data_here["real"].sum() / len(data_here)
+        y_v.append(ratio_ones_here / ratio_ones)
+
+    return list(x_val * 100), list(y_v)
+
+
+
+def calculate_lift_curve(params, data):
+    """Calculate All Lift curves."""
+    res = {}
+    res["lift_train"] = calculate_single_lift_curve(
+        data["label_train"], data["pred_train"]
+    )
+    res["lift_val"] = calculate_single_lift_curve(data["label_val"], data["pred_val"])
+    if "validation_test_split" in params and params["validation_test_split"]:
+        res["lift_test"] = calculate_single_lift_curve(
+            data["label_test"], data["pred_test"]
+        )
+    return res
